
//			FileStamp.cpp


#ifndef FILESTAMP_FUNC_VER
#define FILESTAMP_FUNC_VER				1.0.1.0


#define FILESTAMP_LOCAL					0
#define FILESTAMP_HTTP					1
#define FILESTAMP_FTP					2

#define FILESTAMP_LOCALIZE_BADLY		0
#define FILESTAMP_UTC					1
#define FILESTAMP_LOCALIZE_PROPERLY		2

#define FILESTAMP_LAST_WRITE			0
#define FILESTAMP_LAST_ACCESS			1
#define FILESTAMP_CREATION				2

#define FILESTAMP_FILESYS_UNKNOWN		0
#define FILESTAMP_FILESYS_FAT			1
#define FILESTAMP_FILESYS_NTFS			2



wchar_t FStmp_AgentString[SHORT_BUF_LEN]		= L"";



int FStmp_GetErrorText( int err, LPTSTR outString )
{
	if ( !outString )
		return -1;


	DWORD FM_Flags					= FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS | FORMAT_MESSAGE_MAX_WIDTH_MASK;

	static HMODULE WinINetHandle	= NULL;
	static HMODULE WinHttpHandle	= NULL;


	if ( ( err >= 12000 ) && ( err <= 12199 ) ) {
		if ( !WinINetHandle )
			WinINetHandle = GetModuleHandle( L"wininet.dll" );

		if ( WinINetHandle )
			FM_Flags |= FORMAT_MESSAGE_FROM_HMODULE;
	}


	if ( err == -1 )
		wcscpy_s( outString, SHORT_BUF_LEN, L"Unknown error" );
	else if ( !FormatMessage( FM_Flags, WinINetHandle, err, 0, outString, SHORT_BUF_LEN, NULL ) ) {
		wsprintf( outString, L"Error %d", err );

		if ( ( err >= 12000 ) && ( err <= 12199 ) ) {
			if ( !WinHttpHandle )
				WinHttpHandle = GetModuleHandle( L"winhttp.dll" );

			if ( WinHttpHandle ) {
				FM_Flags |= FORMAT_MESSAGE_FROM_HMODULE;

				if ( !FormatMessage( FM_Flags, WinHttpHandle, err, 0, outString, SHORT_BUF_LEN, NULL ) )
					wsprintf( outString, L"Error %d", err );
			}
		}
	}

#ifdef _WININET_

	if ( err == ERROR_INTERNET_EXTENDED_ERROR ) {
		wchar_t ServerError[SHORT_BUF_LEN]		= L"";
		DWORD ErrorVal							= 0;
		DWORD BufLen							= SHORT_BUF_LEN;

		if ( InternetGetLastResponseInfo( &ErrorVal, ServerError, &BufLen ) ) {
			if ( ServerError[0] ) {
				wcscat_s( outString, SHORT_BUF_LEN, L" : " );
				wcscat_s( outString, SHORT_BUF_LEN, ServerError );

			}
		}
	}

#endif	//	_WININET_

	size_t l = wcslen( outString );

	if ( l > 0 )
		while ( l && iswspace( outString[l - 1] ) ) {
			l--;
			outString[l] = '\0';
		}

	return 0;
}



#ifdef _WIN64

BOOL FStmp_IsWin64()
{
	return TRUE;
}

#else	//	_WIN64

BOOL FStmp_IsWin64()
{
	wchar_t Wow64Dir[MAX_PATH]		= L"";


	GetSystemWow64Directory( Wow64Dir, MAX_PATH );

	if ( Wow64Dir[0] )
		if ( QueryIsDirectory( Wow64Dir ) )
			return TRUE;

	return FALSE;
}

#endif	//	_WIN64



int FStmp_InitializeAgentString()
{
	wchar_t varbuf[SHORT_BUF_LEN]		= L" %_cmdproc/%_4ver (%@if[%_x64 == 1,x64,x86])";

	wsprintf( FStmp_AgentString, L"%s/%u.%u.%u (%s)", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD, PLATFORM );
	ExpandVariables( varbuf, 1 );
	wcscat_s( FStmp_AgentString, SHORT_BUF_LEN, varbuf );

	wsprintf( FStmp_AgentString + wcslen( FStmp_AgentString ), L" Windows/%u.%u.%u (%s)", OSVerInfo.dwMajorVersion, OSVerInfo.dwMinorVersion, OSVerInfo.dwBuildNumber, FStmp_IsWin64() ? L"x64" : L"x86" );
	return 0;
}



int FStmp_INetProtocol( LPCTSTR String )
{
	if ( !String )
		return FILESTAMP_LOCAL;

	if ( !_wcsnicmp( String, L"ftp://", 6 ) )
		return FILESTAMP_FTP;

	if ( !_wcsnicmp( String, L"http://", 7 ) )
		return FILESTAMP_HTTP;

	if ( !_wcsnicmp( String, L"https://", 8 ) )
		return FILESTAMP_HTTP;

	return 0;
}



DWORD FStmp_GetHttpStatus( HINTERNET InternetHandle )
{
	DWORD Status = 0;
	DWORD BufferSize = sizeof( Status );

	HttpQueryInfo( InternetHandle, HTTP_QUERY_STATUS_CODE | HTTP_QUERY_FLAG_NUMBER, &Status, &BufferSize, 0 );

	if ( Status >= 400 )
		return Status;
	else
		return 0;
}



int FStmp_HasWildcards( LPTSTR inString )
{
	int i = 0;
	wchar_t ch = '\0';

	while ( inString[i] ) {
		ch = inString[i];

		if ( ch == '*' )
			return 1;

		if ( ch == '?' ) {
			if ( i != 2 )
				return 1;

			if ( ( inString[0] != '\\' ) || ( inString[1] != '\\' ) || ( inString[3] != '\\' ) )
				return 1;
		}

		i++;
	}

	return 0;
}



int FStmp_LocalFile( LPTSTR Filename, SYSTEMTIME *outStamp, int which, int localize )
{
	if ( !Filename || !outStamp )
		return -1;


	wchar_t Truename[MAX_FILENAME_LEN]			= L"";
	wchar_t RootName[SHORT_BUF_LEN]				= L"";
	wchar_t FSType[SHORT_BUF_LEN]				= L"";

	int filesystem					= FILESTAMP_FILESYS_UNKNOWN;
	int backslashes					= 4;
	int i							= 0;
	wchar_t ch						= '\0';
	BOOL ZeroTime					= FALSE;

	WIN32_FIND_DATA FileInfo;
	FILETIME FileStamp;
	FILETIME FileStampLocal;
	SYSTEMTIME FileStamp2;



	QueryTrueName( arg[0], Truename );    // expands directory aliases!

	if ( FStmp_HasWildcards( Truename ) ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Wildcards not allowed: \"%s\"\r\n", arg[0] );
		return -1;
	}


	//  try to extract the rootname from the filename:

	if ( iswalnum( Truename[0] ) && ( Truename[1] == ':' ) && ( Truename[2] == '\\' ) )
		backslashes = 1;
	else
		backslashes = 4;

	do {
		ch = Truename[i];

		RootName[i++] = ch;
		RootName[i] = '\0';

		if ( ch == '\\' )
			backslashes--;

		if ( i >= ( SHORT_BUF_LEN - 1 ) )
			break;

	}
	while ( ch && backslashes ) ;


	//  try to get the filesystem name:

	if ( !backslashes ) {
		if ( GetVolumeInformation( RootName, NULL, NULL, NULL, NULL, NULL, FSType, SHORT_BUF_LEN ) ) {
			if ( !_wcsnicmp( FSType, L"FAT", 3 ) )
				filesystem = FILESTAMP_FILESYS_FAT;
			else if ( !_wcsnicmp( FSType, L"NTFS", 4 ) || !_wcsicmp( FSType, L"ReFS" ) )
				filesystem = FILESTAMP_FILESYS_NTFS;
		}
	}


	//  get info on the specified file:

	HANDLE FindHandle = FindFirstFile( Truename, &FileInfo );

	if ( FindHandle == INVALID_HANDLE_VALUE ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"File not found: \"%s\"\r\n", arg[0] );
		return -1;
	}

	FindClose( FindHandle );

	switch ( which ) {

	case FILESTAMP_LAST_ACCESS :

		FileStamp = FileInfo.ftLastAccessTime;
		break;

	case FILESTAMP_CREATION :

		FileStamp = FileInfo.ftCreationTime;
		break;

	default :

		FileStamp = FileInfo.ftLastWriteTime;
	}

	if ( ( which == FILESTAMP_LAST_ACCESS ) && ( filesystem == FILESTAMP_FILESYS_FAT ) )
		ZeroTime = TRUE;


	//  localize file time as directed:

	switch ( localize ) {

	case FILESTAMP_LOCALIZE_BADLY :
		//  Bad, broken, backwards-compatible

		FileTimeToLocalFileTime( &FileStamp, &FileStampLocal );
		FileTimeToSystemTime( &FileStampLocal, outStamp );
		break;

	case 1 :
		//  UTC

		FileTimeToSystemTime( &FileStamp, outStamp );
		break;

	default :
		//  Localized correctly

		if ( filesystem == FILESTAMP_FILESYS_NTFS ) {
			FileTimeToSystemTime( &FileStamp, &FileStamp2 );
			SystemTimeToTzSpecificLocalTime( NULL, &FileStamp2, outStamp );
		}
		else {
			FileTimeToLocalFileTime( &FileStamp, &FileStampLocal );
			FileTimeToSystemTime( &FileStampLocal, outStamp );
		}
	}

	if ( ZeroTime )
		outStamp->wHour = outStamp->wMinute = outStamp->wSecond = outStamp->wMilliseconds = 0;

	return 0;
}



int FStmp_GetHttpFileTime( HINTERNET Connect, LPTSTR Url, SYSTEMTIME *TimeStamp, int localize )
{
	if ( !Connect || !TimeStamp )
		return 0;


	HINTERNET RemoteFileHandle		= NULL;
	DWORD OpenUrlFlags				= INTERNET_FLAG_NO_CACHE_WRITE | INTERNET_FLAG_RELOAD | INTERNET_FLAG_PASSIVE;
	DWORD HttpStatus				= 0;
	SYSTEMTIME FileTime;
	DWORD buflen					= sizeof( SYSTEMTIME );
	int rv							= 0;


	RemoteFileHandle = InternetOpenUrl( Connect, Url, NULL, 0, OpenUrlFlags, NULL );

	if ( RemoteFileHandle ) {
		HttpStatus = FStmp_GetHttpStatus( RemoteFileHandle );

		if ( HttpStatus >= 300 )
			rv = 0;
		else if ( !HttpQueryInfoW( RemoteFileHandle, HTTP_QUERY_LAST_MODIFIED | HTTP_QUERY_FLAG_SYSTEMTIME, &FileTime, &buflen, 0 ) )
			rv = 0;
		else {
			if ( localize == FILESTAMP_UTC )
				*TimeStamp = FileTime;
			else
				SystemTimeToTzSpecificLocalTime( NULL, &FileTime, TimeStamp );

			rv = 1;
		}

		InternetCloseHandle( RemoteFileHandle );
	}

	return rv;
}



int FStmp_GetFtpFileTime( HINTERNET Connect, LPTSTR Url, SYSTEMTIME *TimeStamp, int localize )
{
	if ( !Connect || !Url || !TimeStamp )
		return 0;

	if ( Url[0] == '\0' )
		return 0;


	HINTERNET FtpConnection		= NULL;
	HINTERNET SearchHandle		= NULL;
	WIN32_FIND_DATA Found;
	SYSTEMTIME TimeStamp2;
	BOOL Success				= FALSE;

	URL_COMPONENTS Components;

	wchar_t Scheme[SHORT_BUF_LEN]			= L"";
	wchar_t HostName[MAX_ARG_LEN]			= L"";
	wchar_t UrlPath[MAX_FILENAME_LEN]		= L"";
	wchar_t ExtraInfo[MAX_ARG_LEN]			= L"";
	wchar_t UserName[SHORT_BUF_LEN]			= L"";
	wchar_t Password[SHORT_BUF_LEN]			= L"";


	memset( &Components, 0, sizeof( URL_COMPONENTS ) );
	Components.dwStructSize			= sizeof( URL_COMPONENTS );
	Components.lpszScheme			= Scheme;
	Components.dwSchemeLength		= SHORT_BUF_LEN;
	Components.lpszHostName			= HostName;
	Components.dwHostNameLength		= MAX_ARG_LEN;
	Components.lpszUrlPath			= UrlPath;
	Components.dwUrlPathLength		= MAX_FILENAME_LEN;
	Components.lpszExtraInfo		= ExtraInfo;
	Components.dwExtraInfoLength	= MAX_ARG_LEN;
	Components.lpszUserName			= UserName;
	Components.dwUserNameLength		= SHORT_BUF_LEN;
	Components.lpszPassword			= Password;
	Components.dwPasswordLength		= SHORT_BUF_LEN;

	InternetCrackUrl( Url, 0, 0, &Components );


	if ( ( UrlPath[0] == '\0' ) || !wcscmp( UrlPath, L"/" ) )
		return 0;


	FtpConnection = InternetConnect( Connect, HostName, Components.nPort, UserName[0] ? UserName : NULL, ( UserName[0] && Password[0] ) ? Password : NULL, INTERNET_SERVICE_FTP, INTERNET_FLAG_PASSIVE, NULL );

	if ( FtpConnection ) {
		SearchHandle = FtpFindFirstFileW( FtpConnection, UrlPath, &Found, 0, NULL );

		if ( SearchHandle ) {
			Success = TRUE;
			InternetCloseHandle( SearchHandle );
		}

		InternetCloseHandle( FtpConnection );
	}


	if ( Success ) {
		if ( localize == FILESTAMP_UTC ) {
			FileTimeToSystemTime( &( Found.ftLastWriteTime ), &TimeStamp2 );
			TzSpecificLocalTimeToSystemTime( NULL, &TimeStamp2, TimeStamp );
		}
		else
			FileTimeToSystemTime( &( Found.ftLastWriteTime ), TimeStamp );

		return 1;
	}

	return 0;
}



int FStmp_INetFile( LPTSTR Url, SYSTEMTIME *outStamp, int localize )
{
	if ( !Url || !outStamp )
		return -1;


	wchar_t ErrorText[SHORT_BUF_LEN]		= L"";

	int UrlProtocol				= 0;
	HINTERNET Connection		= NULL;
	SYSTEMTIME TimeStamp;
	BOOL Success				= FALSE;


	UrlProtocol = FStmp_INetProtocol( Url );
	if ( !UrlProtocol ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Bad or missing protocol: \"%s\"\r\n", Url );
		return -1;
	}

	if ( FStmp_HasWildcards( Url ) ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Wildcards not allowed: \"%s\"\r\n", arg[0] );
		return -1;
	}


	if ( FStmp_AgentString[0] == '\0' )
		FStmp_InitializeAgentString();


	Connection = InternetOpen( FStmp_AgentString, INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0 );

	if ( Connection ) {

		if ( UrlProtocol == FILESTAMP_HTTP )
			Success = FStmp_GetHttpFileTime( Connection, Url, &TimeStamp, localize );
		else if ( UrlProtocol == FILESTAMP_FTP )
			Success = FStmp_GetFtpFileTime( Connection, Url, &TimeStamp, localize );

		InternetCloseHandle( Connection );
	}
	else {
		FStmp_GetErrorText( GetLastError(), ErrorText );
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Error opening internet: \"%s\"\r\n", ErrorText );
		return -1;
	}

	if ( Success ) {
		*outStamp = TimeStamp;
		return 0;
	}
	else
		return -1;
}



DLLExports int WINAPI f_filestamp( LPTSTR String )
{
	if ( !String )
		return 1;


	wchar_t temp[SHORT_BUF_LEN]			= L"";

	int fmt					= 4;
	int which				= -1;
	int localize			= -1;
	int flags				= 0;
	int protocol			= FILESTAMP_LOCAL;

	wchar_t ch				= '\0';
	wchar_t DateSep			= '@';
	BOOL ShowSeconds		= TRUE;
	BOOL ShowZed			= TRUE;

	SYSTEMTIME TimeStamp;
	int rv					= 0;


	ParseArgs( String, PARSE_BREAK_SPACES | PARSE_BREAK_COMMAS );
	GetLocalDateFormat();


	if ( arglen[0] == 0 ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"No filename\r\n" );
		return -1;
	}

	if ( arglen[1] ) {
		for ( int i = 0; arg[1][i]; i++ ) {
			ch = towupper( arg[1][i] );

			if ( ( ch == 'W' ) && ( which < 0 ) )				//  return last-write stamp
				which = FILESTAMP_LAST_WRITE;
			else if ( ( ch == 'A' ) && ( which < 0 ) )			//  return last-access stamp
				which = FILESTAMP_LAST_ACCESS;
			else if ( ( ch == 'C' ) && ( which < 0 ) )			//  return creation stamp
				which = FILESTAMP_CREATION;
			else if ( ( ch == 'B' ) && ( localize < 0 ) )		//  do bad, broken localization
				localize = FILESTAMP_LOCALIZE_BADLY;
			else if ( ( ch == 'U' ) && ( localize < 0 ) )		//  return utc time
				localize = FILESTAMP_UTC;
			else if ( ( ch == 'L' ) && ( localize < 0 ) )		//  do localization correctly
				localize = FILESTAMP_LOCALIZE_PROPERLY;
			else if ( ch ==  'S' )
				;
			else {
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Invalid flags: \"%s\"\r\n", arg[1] );
				return -1;
			}
		}
	}

	if ( arglen[2] ) {
		fmt = ParseInt( arg[2] );

		if ( fmt < 0 ) {
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Bad format: \"%s\"\r\n", arg[2] );
			return -1;
		}
	}

	if ( arglen[3] ) {
		flags = ParseInt( arg[3] );

		if ( flags < 0 ) {
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Bad option: \"%s\"\r\n", arg[3] );
			return -1;
		}
	}

	if ( which < 0 )
		which = FILESTAMP_LAST_WRITE;

	if ( localize < 0 )
		localize = FILESTAMP_LOCALIZE_PROPERLY;

	if ( flags & 2 )
		DateSep = 0x20;
	else if ( flags & 1 )
		DateSep = 'T';
	else
		DateSep = '@';

	if ( flags & 16 )
		ShowSeconds = FALSE;

	if ( flags & 32 )
		ShowZed = FALSE;


	protocol = FStmp_INetProtocol( arg[0] );

	if ( protocol )
		rv = FStmp_INetFile( arg[0], &TimeStamp, localize );
	else
		rv = FStmp_LocalFile( arg[0], &TimeStamp, which, localize );

	if ( rv )
		return rv;


	//  Format date and time stamps properly:

	if ( ( TimeStamp.wYear < Epoch ) || ( TimeStamp.wYear > Ultima ) ) {
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Date out of range (year = %u)\r\n", TimeStamp.wYear );
		return -1;
	}


	OutputFormattedDate( TimeStamp, fmt, temp );

	if ( fmt == 276 ) {
		if ( ShowSeconds )
			wsprintf( String, L"%s%02u%02u%02u", temp, TimeStamp.wHour, TimeStamp.wMinute, TimeStamp.wSecond );
		else
			wsprintf( String, L"%s%02u%02u", temp, TimeStamp.wHour, TimeStamp.wMinute );
	}
	else {
		if ( ShowSeconds )
			wsprintf( String, L"%s%c%02u:%02u:%02u", temp, DateSep, TimeStamp.wHour, TimeStamp.wMinute, TimeStamp.wSecond );
		else
			wsprintf( String, L"%s%c%02u:%02u", temp, DateSep, TimeStamp.wHour, TimeStamp.wMinute );
	}

	if ( ( localize == FILESTAMP_UTC ) && ShowZed )
		wcscat_s( String, MAX_ARG_LEN, L"Z" );

	return 0;
}


#endif	//	FILESTAMP_FUNC_VER



/*

	1.0.0.0		2023-01-11
				Rewritten to support ftp:// and http:// files.

	1.0.1.0		2023-02-13
				Updated FStmp_GetErrorText() with improvements from Acquire.  Tweaked
				the user agent string to include the plugin's platform string.

*/
