#ifndef POPCAL
#define POPCAL		1.5.1.1


#define POPCAL_BOX_DISABLED				0x0100
#define POPCAL_BOX_SUNDAY				0x0200
#define POPCAL_BOX_SELECTED				0x0400
#define POPCAL_BOX_HOLIDAY				0x0800

#define POPCAL_FLAG_NO_CANCEL			0x0002
#define POPCAL_FLAG_RESTRICT			0x0004
#define POPCAL_FLAG_PREFER_EARLIER		0x0010
#define POPCAL_FLAG_PREFER_LATER		0x0020
#define POPCAL_FLAG_PREFER_MASK			0x0030
#define POPCAL_FLAG_HIDE_HOLIDAYS		0x0080
#define POPCAL_FLAG_DISABLE_HOLIDAYS	0x0100
#define POPCAL_FLAG_USE_ENGLISH			0x0200

#define POPCAL_NUM_BOXES				37



struct _PopCalDate {
	unsigned int year;
	unsigned int month;
	unsigned int day;
};


struct _PopCalVars {
	_PopCalDate Selected;
	_PopCalDate View;
	_PopCalDate InitialDate;
	_PopCalDate MinDate;
	_PopCalDate MaxDate;

	int Flags;
	int DowMask;
	int Countdown;
	BOOL DoCountdown;
	BOOL KeystrokeEndsCountdown;

	HWND hWnd;
	UINT_PTR TimerID;
	LPTSTR Caption;
	HFONT HeaderFont;
	HFONT DateFont;
	HFONT DisabledFont;
	HFONT SelectedFont;
	HFONT SmallFont;
	HFONT ButtonFont;
	HBRUSH SelectedBoxBgnd;
	COLORREF SundayTextColor;
	COLORREF HolidayTextColor;
	COLORREF DisabledColor;
	HHOOK KeyboardHook;

	int Box[POPCAL_NUM_BOXES];

	int HolidaysYear;
	int HolidaysYear2;
	unsigned short int Holidays[12][31];
	unsigned short int Holidays2[12][31];
};


_PopCalVars *PopCalVars		= NULL;




long int Julian( _PopCalDate Date )
{
	return Julian( Date.year, Date.month, Date.day );
}



int OutputFormattedDate( LPTSTR outString, _PopCalDate Date, int fmt )
{
	SYSTEMTIME SysDate;

	SysDate.wYear	= Date.year;
	SysDate.wMonth	= Date.month;
	SysDate.wDay	= Date.day;

	return OutputFormattedDate( SysDate, fmt, outString );
}



int MakeDate( long int julian, _PopCalDate &Date )
{
	SYSTEMTIME SysDate;

	int rv			= 0;


	if ( rv = MakeDate( julian, SysDate ) )
		return rv;

	Date.year	= SysDate.wYear;
	Date.month	= SysDate.wMonth;
	Date.day	= SysDate.wDay;

	return rv;
}



int ParseDate( LPTSTR String, _PopCalDate &Date, int fmt )
{
	SYSTEMTIME SysDate;

	int rv			= 0;


	if ( rv = ParseDate( String, fmt, SysDate ) )
		return rv;

	Date.year	= SysDate.wYear;
	Date.month	= SysDate.wMonth;
	Date.day	= SysDate.wDay;

	return rv;
}



int PopCalCenterDialog( HWND hwnd )
{
	RECT ConsoleRect, DialogRect;
	long int ConsoleWidth, ConsoleHeight, DialogWidth, DialogHeight;

	GetWindowRect( GetConsoleWindow(), &ConsoleRect );
	GetWindowRect( hwnd, &DialogRect );

	ConsoleWidth	= ConsoleRect.right - ConsoleRect.left;
	ConsoleHeight	= ConsoleRect.bottom - ConsoleRect.top;
	DialogWidth		= DialogRect.right - DialogRect.left;
	DialogHeight	= DialogRect.bottom - DialogRect.top;
	
	SetWindowPos( hwnd, HWND_TOP, ConsoleRect.left + ( ConsoleWidth - DialogWidth ) / 2, ConsoleRect.top + ( ConsoleHeight - DialogHeight ) / 2, DialogWidth, DialogHeight, SWP_SHOWWINDOW );
	return 0;
}



int RgbLum( COLORREF Color )
{
	int r		= ( Color & 0x0000ff );
	int g		= ( Color & 0x00ff00 ) >> 8;
	int b		= ( Color & 0xff0000 ) >> 16;

	return ( r * 299 / 255 ) + ( g * 587 / 255 ) + ( b * 114 / 255 );
}



BOOL DrawStatusLine( HWND hwnd )
{
	wchar_t statusbuf[SHORT_BUF_LEN];

	long int julian			= 0;
	_HolidayDef *This		= FirstHolidayDef;
	_HolidayDef *Found		= NULL;


	OutputFormattedDate( statusbuf, PopCalVars->Selected, ( PopCalVars->Flags & POPCAL_FLAG_USE_ENGLISH ) ? 17 : 32 );

	if ( FirstHolidayDef && ( ( PopCalVars->Flags & POPCAL_FLAG_HIDE_HOLIDAYS ) == 0 ) && ( PopCalVars->Selected.year == PopCalVars->HolidaysYear ) ) {
		julian	= Julian( PopCalVars->Selected );

		while ( This ) {
			if ( This->Enabled && ( This->Julian == julian ) && This->Name[0] ) {
				Found = This;
				break;
			}

			This = This->Next;
		}

		if ( Found ) {
			wcscat_s( statusbuf, SHORT_BUF_LEN, L"\n" );
			wcscat_s( statusbuf, SHORT_BUF_LEN, This->Name );		
		}
	}
	
	SetDlgItemText( hwnd, STATUSLINE, statusbuf );
	return FALSE;
}



BOOL FillDowLine( HWND hwnd )
{
	for ( int i = 0; i < 7; i++ ) {
		SetDlgItemText( hwnd, DOWBOX0 + i, ( PopCalVars->Flags & POPCAL_FLAG_USE_ENGLISH ) ? DowName[( i + LeftColumnDow ) % 7] : LocalDowName[( i + LeftColumnDow ) % 7] );
		SendDlgItemMessage( hwnd, DOWBOX0 + i, WM_SETFONT, (WPARAM) PopCalVars->SmallFont, TRUE );
	}

	return 0;
}



BOOL DrawPopCalendar( HWND hwnd )
{
	wchar_t shortbuf[MAX_ARG_LEN + 1];

	long julian	= Julian( PopCalVars->View.year, PopCalVars->View.month, 1 );
	BOOL leap	= Leapness( PopCalVars->View.year );
	int b, boxnum;
	BOOL atStart, atEnd;


	if ( ( PopCalVars->View.year != PopCalVars->HolidaysYear ) && ( ( PopCalVars->Flags & POPCAL_FLAG_HIDE_HOLIDAYS ) == 0 ) && FirstHolidayDef ) {
		PopCalVars->HolidaysYear = PopCalVars->View.year;
		ExpandHolidays( PopCalVars->HolidaysYear, PopCalVars->Holidays );
	}

	b = ( julian + 7 - LeftColumnDow ) % 7;

	for ( int i = 0; i < POPCAL_NUM_BOXES; i++ ) {
		boxnum = ( i - b + 1 );
		if ( ( boxnum < 1 ) || ( boxnum > MonthLength[PopCalVars->View.month][leap] ) ) {
			PopCalVars->Box[i] = 0;
			ShowWindow( GetDlgItem(hwnd, DATEBOX0 + i), FALSE );
		}
		else if ( ( ( PopCalVars->View.year == PopCalVars->MinDate.year ) && ( PopCalVars->View.month == PopCalVars->MinDate.month ) && ( boxnum < (int) PopCalVars->MinDate.day ) ) ||
				  ( ( PopCalVars->View.year == PopCalVars->MaxDate.year ) && ( PopCalVars->View.month == PopCalVars->MaxDate.month ) && ( boxnum > (int) PopCalVars->MaxDate.day ) ) ||
				  ( PopCalVars->DowMask & ( 1 << ( ( i + LeftColumnDow ) % 7 ) ) ) ||
				  ( ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS ) && PopCalVars->Holidays[PopCalVars->View.month - 1][boxnum - 1] ) ) {
			PopCalVars->Box[i] = boxnum | POPCAL_BOX_DISABLED;
			Sprintf( shortbuf, L"%u", boxnum );
			SetDlgItemText( hwnd, DATEBOX0 + i, shortbuf );
			SendDlgItemMessage( hwnd, DATEBOX0 + i, WM_SETFONT, (WPARAM) PopCalVars->DisabledFont, TRUE );
			ShowWindow( GetDlgItem( hwnd, DATEBOX0 + i ), TRUE );
		}
		else if ( ( boxnum == PopCalVars->Selected.day ) && ( PopCalVars->View.month == PopCalVars->Selected.month ) && ( PopCalVars->View.year == PopCalVars->Selected.year ) ) {
			PopCalVars->Box[i] = boxnum | POPCAL_BOX_SELECTED;
			if ( ( ( i + LeftColumnDow ) % 7 ) == 0 )
				PopCalVars->Box[i] |= POPCAL_BOX_SUNDAY;

			if ( PopCalVars->Holidays[PopCalVars->View.month - 1][boxnum - 1] )
				PopCalVars->Box[i] |= POPCAL_BOX_HOLIDAY;

			Sprintf( shortbuf, L"%u", boxnum );
			SetDlgItemText( hwnd, DATEBOX0 + i, shortbuf );
			SendDlgItemMessage( hwnd, DATEBOX0 + i, WM_SETFONT, (WPARAM) PopCalVars->SelectedFont, TRUE );
			ShowWindow( GetDlgItem( hwnd, DATEBOX0 + i ), TRUE );
		}
		else {
			PopCalVars->Box[i] = boxnum;
			if ( ( ( i + LeftColumnDow ) % 7 ) == 0 )
				PopCalVars->Box[i] |= POPCAL_BOX_SUNDAY;

			if ( PopCalVars->Holidays[PopCalVars->View.month - 1][boxnum - 1] )
				PopCalVars->Box[i] |= POPCAL_BOX_HOLIDAY;

			Sprintf( shortbuf, L"%u", boxnum );
			SetDlgItemText( hwnd, DATEBOX0 + i, shortbuf );
			SendDlgItemMessage( hwnd, DATEBOX0 + i, WM_SETFONT, (WPARAM) PopCalVars->DateFont, TRUE );
			ShowWindow( GetDlgItem( hwnd, DATEBOX0 + i ), TRUE );
		}
	}

	atStart	= ( ( PopCalVars->View.year == PopCalVars->MinDate.year ) && ( PopCalVars->View.month == PopCalVars->MinDate.month ) );
	atEnd	= ( ( PopCalVars->View.year == PopCalVars->MaxDate.year ) && ( PopCalVars->View.month == PopCalVars->MaxDate.month ) );

	EnableWindow( GetDlgItem( hwnd, BACKBUTTON ), !atStart );
	EnableWindow( GetDlgItem( hwnd, YEARBACKBUTTON ), !atStart );
	EnableWindow( GetDlgItem( hwnd, FORWARDBUTTON ), !atEnd );
	EnableWindow( GetDlgItem( hwnd, YEARFORWARDBUTTON ), !atEnd );
	
	Sprintf( shortbuf, L"%s %u", ( PopCalVars->Flags & POPCAL_FLAG_USE_ENGLISH ) ? MonthName[PopCalVars->View.month] : LocalMonthNameLong[PopCalVars->View.month], PopCalVars->View.year );
	SetDlgItemText( hwnd, HEADERLINE, shortbuf );
	return 0;
}



int PopCalTerminateTimer()
{
	if ( PopCalVars->TimerID ) {
		SetDlgItemText( PopCalVars->hWnd, IDOK, L"OK" );
		KillTimer( NULL, PopCalVars->TimerID );
		PopCalVars->TimerID = NULL;
	}

	return 0;
}



int PopCalIsHoliday( long int julian )
{
	if ( ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS ) == 0 )
		return 0;

	if ( FirstHolidayDef == NULL )
		return 0;

	if ( ( julian < MinimumJulian ) || ( julian > MaximumJulian ) )
		return 0;


	SYSTEMTIME Date;


	MakeDate( julian, Date );


	if ( Date.wYear == PopCalVars->HolidaysYear )
		return PopCalVars->Holidays[Date.wMonth - 1][Date.wDay - 1];

	if ( Date.wYear == PopCalVars->HolidaysYear2 )
		return PopCalVars->Holidays2[Date.wMonth - 1][Date.wDay - 1];
	
	PopCalVars->HolidaysYear2 = Date.wYear;
	ExpandHolidays( Date.wYear, PopCalVars->Holidays2 );

	return PopCalVars->Holidays2[Date.wMonth - 1][Date.wDay - 1];
}



BOOL SeekUp( long int &julian )
{
	if ( ( PopCalVars->DowMask & 0x7f ) == 0x7f )
		return FALSE;


	long int j				= julian;
	long int MaxJulian		= Julian( PopCalVars->MaxDate );


	if ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS ) {

		if ( ( ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) ) == 0 ) && !PopCalIsHoliday( j ) )
			return TRUE;

		while ( ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) ) || PopCalIsHoliday( j ) )
			j++;

		if ( j > MaxJulian )
			return FALSE;

		julian = j;
		return TRUE;
	}
	else {

		if ( ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) ) == 0 )
			return TRUE;

		while ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) )
			j++;

		if ( j > MaxJulian )
			return FALSE;

		julian = j;
		return TRUE;
	}
}



BOOL SeekDown( long int &julian )
{
	if ( ( PopCalVars->DowMask & 0x7f ) == 0x7f )
		return FALSE;


	long int j				= julian;
	long int MinJulian		= Julian( PopCalVars->MinDate );


	if ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS ) {

		if ( ( ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) ) == 0 ) && !PopCalIsHoliday( j ) )
			return TRUE;

		while ( ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) ) || PopCalIsHoliday( j ) )
			j--;

		if ( j < MinJulian )
			return FALSE;

		julian = j;
		return TRUE;
	}
	else {

		if ( ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) ) == 0 )
			return TRUE;

		while ( PopCalVars->DowMask & ( 1 << ( j % 7 ) ) )
			j--;

		if ( j < MinJulian )
			return FALSE;

		julian = j;
		return TRUE;
	}
}



BOOL SeekNearest( long int &julian )
{
	long int upjul			= julian;
	long int downjul		= julian;
	BOOL uprv				= FALSE;
	BOOL downrv				= FALSE;


	uprv	= SeekUp( upjul );
	downrv	= SeekDown( downjul );


	if ( !uprv && !downrv )
		return FALSE;
	else if ( uprv && !downrv ) {
		julian	= upjul;
		return TRUE;
	}
	else if ( downrv && !uprv ) {
		julian	= downrv;
		return TRUE;
	}


	if ( ( upjul - julian ) < ( julian - downjul ) )
		julian	= upjul;
	else if ( ( upjul - julian ) > ( julian - downjul ) )
		julian	= downjul;
	else if ( ( julian % 7 ) > 3 )
		julian	= downjul;
	else
		julian	= upjul;

	return TRUE;
}



BOOL BackMonth( HWND hwnd )
{
	unsigned int trymonth = PopCalVars->View.month;
	unsigned int tryyear = PopCalVars->View.year;

	trymonth--;
	if ( trymonth < 1 ) {
		trymonth = 12;
		tryyear--;
	}

	if ( tryyear < PopCalVars->MinDate.year )
		return FALSE;

	if ( ( tryyear == PopCalVars->MinDate.year ) && ( trymonth < PopCalVars->MinDate.month ) )
		return FALSE;

	PopCalVars->View.month = trymonth;
	PopCalVars->View.year = tryyear;
	DrawPopCalendar( hwnd );
	return TRUE;
}



BOOL AheadMonth( HWND hwnd )
{
	unsigned int trymonth = PopCalVars->View.month;
	unsigned int tryyear = PopCalVars->View.year;

	trymonth++;
	if ( trymonth > 12 ) {
		trymonth = 1;
		tryyear++;
	}

	if ( tryyear > PopCalVars->MaxDate.year )
		return FALSE;

	if ( ( tryyear == PopCalVars->MaxDate.year ) && ( trymonth > PopCalVars->MaxDate.month ) )
		return FALSE;

	PopCalVars->View.month = trymonth;
	PopCalVars->View.year = tryyear;
	DrawPopCalendar( hwnd );
	return TRUE;
}



BOOL BackYear( HWND hwnd )
{
	unsigned int tryyear = PopCalVars->View.year;
	unsigned int trymonth = PopCalVars->View.month;

	if ( ( tryyear == PopCalVars->MinDate.year ) && ( trymonth == PopCalVars->MinDate.month ) )
		return FALSE;

	if ( tryyear == PopCalVars->MinDate.year )
		trymonth = PopCalVars->MinDate.month;
	else {
		tryyear--;
		if ( ( tryyear == PopCalVars->MinDate.year ) && ( trymonth < PopCalVars->MinDate.month ) )
			trymonth = PopCalVars->MinDate.month;
	}

	PopCalVars->View.year  = tryyear;
	PopCalVars->View.month = trymonth;

	DrawPopCalendar( hwnd );
	return TRUE;
}


BOOL AheadYear( HWND hwnd )
{
	unsigned int tryyear = PopCalVars->View.year;
	unsigned int trymonth = PopCalVars->View.month;

	if ( ( tryyear == PopCalVars->MaxDate.year ) && ( trymonth == PopCalVars->MaxDate.month ) )
		return FALSE;

	if ( tryyear == PopCalVars->MaxDate.year )
		trymonth = PopCalVars->MaxDate.month;
	else {
		tryyear++;
		if ( ( tryyear == PopCalVars->MaxDate.year ) && ( trymonth > PopCalVars->MaxDate.month ) )
			trymonth = PopCalVars->MaxDate.month;
	}

	PopCalVars->View.year  = tryyear;
	PopCalVars->View.month = trymonth;

	DrawPopCalendar( hwnd );
	return TRUE;
}



BOOL SelBackDay( HWND hwnd )
{
	long int julian = Julian( PopCalVars->Selected ) - 1;
	if ( julian < Julian( PopCalVars->MinDate ) )
		return FALSE;

	if ( !SeekDown( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelAheadDay( HWND hwnd )
{
	long int julian = Julian( PopCalVars->Selected ) + 1;
	if ( julian > Julian( PopCalVars->MaxDate ) )
		return FALSE;

	if ( !SeekUp( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelBackWeek( HWND hwnd )
{
	long int julian = Julian( PopCalVars->Selected ) - 7;
	if ( julian < Julian( PopCalVars->MinDate ) )
		return FALSE;

	if ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS )
		if ( !SeekNearest( julian ) )
			return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelAheadWeek( HWND hwnd )
{
	long int julian = Julian( PopCalVars->Selected ) + 7;
	if ( julian > Julian( PopCalVars->MaxDate ) )
		return FALSE;

	if ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS )
		if ( !SeekNearest( julian ) )
			return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelBackMonth( HWND hwnd )
{
	int tryyear  = PopCalVars->Selected.year;
	int trymonth = PopCalVars->Selected.month;
	int tryday   = PopCalVars->Selected.day;

	if ( ( tryyear == PopCalVars->MinDate.year ) && ( trymonth == PopCalVars->MinDate.month ) && ( tryday == PopCalVars->MinDate.day ) )
		return FALSE;

	long int kjul = Julian( tryyear, trymonth, 1 );
	long int ljul = kjul;
	if ( !SeekUp( ljul ) )
		return FALSE;

	int first = ljul - kjul + 1;

	if ( ( ( tryyear != PopCalVars->MinDate.year ) || ( trymonth != PopCalVars->MinDate.month ) ) && ( tryday == first ) ) {
		trymonth--;
		if ( trymonth < 1 ) {
			tryyear--;
			trymonth = 12;
		}
		tryday = 1;
	}
	else if ( ( tryyear != PopCalVars->MinDate.year ) || ( trymonth != PopCalVars->MinDate.month ) )
		tryday = first;
	else
		tryday = PopCalVars->MinDate.day;

	long int julian = Julian( tryyear, trymonth, tryday );

	if ( !SeekUp( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelAheadMonth( HWND hwnd )
{
	int tryyear  = PopCalVars->Selected.year;
	int trymonth = PopCalVars->Selected.month;
	int tryday   = PopCalVars->Selected.day;

	if ( ( tryyear == PopCalVars->MaxDate.year ) && ( trymonth == PopCalVars->MaxDate.month ) && ( tryday == PopCalVars->MaxDate.day ) )
		return FALSE;

	if ( ( tryyear != PopCalVars->MaxDate.year ) || ( trymonth != PopCalVars->MaxDate.month ) ) {
		trymonth++;
		if ( trymonth > 12 ) {
			tryyear++;
			trymonth = 1;
		}
		tryday = 1;
	}
	else
		tryday = PopCalVars->MaxDate.day;

	long int julian = Julian( tryyear, trymonth, tryday );

	if ( ( tryyear == PopCalVars->MaxDate.year ) && ( trymonth == PopCalVars->MaxDate.month ) && ( tryday > 15 ) ) {
		if ( !SeekDown( julian ) )
			return FALSE;
	}
	else {
		if ( !SeekUp( julian ) )
			return FALSE;
	}

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelBackYear( HWND hwnd )
{
	unsigned int tryyear  = PopCalVars->Selected.year;
	unsigned int trymonth = PopCalVars->Selected.month;
	unsigned int tryday   = PopCalVars->Selected.day;

	if ( ( tryyear == PopCalVars->MinDate.year ) && ( trymonth == PopCalVars->MinDate.month ) && ( tryday == PopCalVars->MinDate.day ) )
		return FALSE;

	long int kjul = Julian( tryyear, trymonth, 1 );
	long int ljul = kjul;
	if ( !SeekUp( ljul ) )
		return FALSE;

	int first = ljul - kjul + 1;

	if ( ( tryyear > PopCalVars->MinDate.year ) && ( trymonth == 1 ) && ( tryday == first ) ) {
		tryyear--;
		tryday = 1;
		if ( tryyear == PopCalVars->MinDate.year ) {
			trymonth = PopCalVars->MinDate.month;
			tryday = PopCalVars->MinDate.day;
		}
	}
	else if ( tryyear > PopCalVars->MinDate.year ) {
		trymonth = 1;
		tryday = 1;
	}
	else {
		trymonth = PopCalVars->MinDate.month;
		tryday = PopCalVars->MinDate.day;
	}

	long int julian = Julian( tryyear, trymonth, tryday );

	if ( !SeekUp( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelAheadYear( HWND hwnd )
{
	unsigned int tryyear  = PopCalVars->Selected.year;
	unsigned int trymonth = PopCalVars->Selected.month;
	unsigned int tryday   = PopCalVars->Selected.day;

	if ( ( tryyear == PopCalVars->MaxDate.year ) && ( trymonth == PopCalVars->MaxDate.month ) && ( tryday == PopCalVars->MaxDate.day ) )
		return FALSE;

	if ( tryyear < PopCalVars->MaxDate.year ) {
		tryyear++;
		trymonth = 1;
		tryday = 1;
	}
	else {
		trymonth = PopCalVars->MaxDate.month;
		tryday = PopCalVars->MaxDate.day;
	}

	long int julian = Julian( tryyear, trymonth, tryday );

	if ( ( tryyear == PopCalVars->MaxDate.year) && (trymonth == PopCalVars->MaxDate.month ) ) {
		if ( !SeekDown( julian ) )
			return FALSE;
	}
	else {
		if ( !SeekUp( julian ) )
			return FALSE;
	}

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelWeekTop( HWND hwnd )
{
	long int julian = Julian( PopCalVars->Selected );
	long int minjul = Julian( PopCalVars->MinDate );
	long int maxjul = Julian( PopCalVars->MaxDate );

	long int t = Julian( PopCalVars->Selected.year, PopCalVars->Selected.month, 1 );
	if ( minjul < t )
		minjul = t;

	t = t + MonthLength[PopCalVars->Selected.month][Leapness( PopCalVars->Selected.year )] - 1;
	if ( maxjul > t )
		maxjul = t;

	int col = ( julian - LeftColumnDow + 7 ) % 7;

	julian -= col;

	if ( julian < minjul )
		julian = minjul;

	if ( !SeekUp( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelWeekEnd( HWND hwnd )
{
	long int julian = Julian( PopCalVars->Selected );
	long int minjul = Julian( PopCalVars->MinDate );
	long int maxjul = Julian( PopCalVars->MaxDate );

	long int t = Julian( PopCalVars->Selected.year, PopCalVars->Selected.month, 1 );
	if ( minjul < t )
		minjul = t;

	t = t + MonthLength[PopCalVars->Selected.month][Leapness( PopCalVars->Selected.year )] - 1;
	if ( maxjul > t )
		maxjul = t;

	int col = ( julian - LeftColumnDow + 7 ) % 7;

	julian += ( 6 - col );

	if ( julian > maxjul )
		julian = maxjul;

	if ( !SeekDown( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );

	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelMinimum( HWND hwnd )
{
	long int julian = Julian( PopCalVars->MinDate );
	if ( !SeekUp( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelMaximum( HWND hwnd )
{
	long int julian = Julian( PopCalVars->MaxDate );
	if ( !SeekDown( julian ) )
		return FALSE;

	MakeDate( julian, PopCalVars->Selected );
	PopCalVars->View = PopCalVars->Selected;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL SelInit( HWND hwnd )
{
	PopCalVars->Selected = PopCalVars->InitialDate;
	PopCalVars->View = PopCalVars->InitialDate;
	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL ChangeSelection( HWND hwnd, int boxnum )
{
	if ( PopCalVars->Box[boxnum] == 0 )
		return FALSE;

	if ( PopCalVars->Box[boxnum] & POPCAL_BOX_DISABLED )
		return FALSE;

	PopCalVars->Selected.year  = PopCalVars->View.year;
	PopCalVars->Selected.month = PopCalVars->View.month;
	PopCalVars->Selected.day   = PopCalVars->Box[boxnum] & 31;

	DrawPopCalendar( hwnd );
	DrawStatusLine( hwnd );
	return TRUE;
}



BOOL QuickExit( HWND hwnd, int boxnum )
{
	if ( ChangeSelection( hwnd, boxnum ) )
		EndDialog( hwnd, IDOK );

	return FALSE;
}



BOOL ChangeLeftColumn(HWND hwnd)
{
	LeftColumnDow = (LeftColumnDow + 1) % 7;
	
	FillDowLine(hwnd);
	DrawPopCalendar(hwnd);

	return TRUE;
}


int PopCalEndCountdown()
{
	PopCalVars->DoCountdown	= FALSE;
	PopCalVars->Countdown	= 0;
	SetDlgItemText( PopCalVars->hWnd, IDOK, L"OK" );
	return 0;
}



LRESULT CALLBACK PopCalKeyboardProc( int code, WPARAM wParam, LPARAM lParam )
{
	if ( code < 0 )
		return CallNextHookEx( PopCalVars->KeyboardHook, code, wParam, lParam );

	if ( PopCalVars->KeystrokeEndsCountdown )
		PopCalEndCountdown();

	switch ( wParam ) {

	case VK_LEFT :
		if ( !( lParam & 0x80000000 ) )
			PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTBACKDAY | 0x10000, 0 );

		return 1;

	case VK_RIGHT :
		if ( !( lParam & 0x80000000 ) )
			PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTAHEADDAY | 0x10000, 0 );

		return 1;

	case VK_UP :
		if ( !( lParam & 0x80000000 ) )
			PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTBACKWEEK | 0x10000, 0 );

		return 1;

	case VK_DOWN :
		if ( !( lParam & 0x80000000 ) )
			PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTAHEADWEEK | 0x10000, 0 );

		return 1;

	case VK_PRIOR :
		if ( !( lParam & 0x80000000 ) ) {
			if ( GetKeyState( VK_CONTROL ) & 0x8000 )
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTBACKYEAR | 0x10000, 0 );
			else
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTBACKMONTH | 0x10000, 0 );
		}
		return 1;

	case VK_NEXT :
		if ( !( lParam & 0x80000000 ) ) {
			if ( GetKeyState( VK_CONTROL ) & 0x8000 )
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTAHEADYEAR | 0x10000, 0 );
			else
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTAHEADMONTH | 0x10000, 0 );
		}
		return 1;

	case VK_HOME :
		if ( !( lParam & 0x80000000 ) ) {
			if ( GetKeyState( VK_CONTROL ) & 0x8000 )
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTMINIMUM | 0x10000, 0 );
			else
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTWEEKTOP | 0x10000, 0 );
		}
		return 1;

	case VK_END :
		if ( !( lParam & 0x80000000 ) ) {
			if ( GetKeyState( VK_CONTROL ) & 0x8000 )
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTMAXIMUM | 0x10000, 0 );
			else
				PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTWEEKEND | 0x10000, 0 );
		}
		return 1;
		
	case VK_BACK :
		if ( !( lParam & 0x80000000 ) )
			PostMessage( PopCalVars->hWnd, WM_COMMAND, SELECTINITIAL | 0x10000, 0 );

		return 1;

	case VK_PAUSE :
		if ( !( lParam & 0x80000000 ) )
			if ( GetKeyState( VK_SHIFT ) & 0x8000 )
				PostMessage( PopCalVars->hWnd, WM_COMMAND, OTHERBUTTON | 0x10000, 0 );

		return 1;

	case VK_ADD :
		if ( !( lParam & 0x80000000 ) )
			PostMessage( PopCalVars->hWnd, WM_COMMAND, STATUSLINE, 0 );

		return 1;

	default:
		return CallNextHookEx( PopCalVars->KeyboardHook, code, wParam, lParam );
	}
}



VOID CALLBACK PopCalTimerProc( HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime )
{
	wchar_t shortbuf[SHORT_BUF_LEN];


	PopCalVars->KeystrokeEndsCountdown = TRUE;

	if ( !PopCalVars->DoCountdown )
		return;

	if ( PopCalVars->Countdown == 0 )
		EndDialog( PopCalVars->hWnd, IDOK );

	PopCalVars->Countdown--;
	Sprintf( shortbuf, L"OK - %d", PopCalVars->Countdown );
	SetDlgItemText( PopCalVars->hWnd, IDOK, shortbuf );
}



int PopCalDeleteObjects()
{
	DeleteObject( PopCalVars->HeaderFont );
	DeleteObject( PopCalVars->DateFont );
	DeleteObject( PopCalVars->SelectedFont );
	DeleteObject( PopCalVars->DisabledFont );
	DeleteObject( PopCalVars->SmallFont );
	DeleteObject( PopCalVars->ButtonFont );
	DeleteObject( PopCalVars->SelectedBoxBgnd );

	if ( PopCalVars->KeyboardHook )
		UnhookWindowsHookEx( PopCalVars->KeyboardHook );
	
	PopCalTerminateTimer();
	return 0;
}



BOOL PopCalInitializeDialog( HWND hwnd )
{
	HDC Desktop			= NULL;
	int LogPixelsY		= 72;
	long lfHeight1, lfHeight2, lfHeight3, lfHeight4;
	HICON CalDateIcon	= NULL;


	PopCalVars->hWnd = hwnd;

	CalDateIcon = LoadIcon( MyDLLHandle, L"CALDATE_ICON" );
	if ( CalDateIcon )
		SendMessage( hwnd, WM_SETICON, ICON_BIG, (LPARAM) CalDateIcon );

	if ( PopCalVars->Caption ) {
		SetWindowText( hwnd, PopCalVars->Caption );
		free( PopCalVars->Caption );
		PopCalVars->Caption = NULL;
	}

	PopCalCenterDialog( hwnd );

	Desktop	= GetDC( NULL );
	if ( Desktop ) {
		LogPixelsY	= GetDeviceCaps( Desktop, LOGPIXELSY );
		ReleaseDC( NULL, Desktop );
	}

	//  Calculate heights for the various fonts used:
	lfHeight1 = -MulDiv( 16, LogPixelsY, 72 );  // big
	lfHeight2 = -MulDiv( 12, LogPixelsY, 72 );  // medium
	lfHeight3 = -MulDiv( 9, LogPixelsY, 72 );   // small

	if ( ( ( PopCalVars->Flags & POPCAL_FLAG_USE_ENGLISH ) > 0 ) || ( ( GetUserDefaultLangID() & 0x00ff ) == 0x09 ) )
		lfHeight4 = -MulDiv( 16, LogPixelsY, 72 ) ;  // super big for English
	else
		lfHeight4 = -MulDiv( 12, LogPixelsY, 72 );  // not so big for non-English


	PopCalVars->HeaderFont = CreateFont( lfHeight4, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, L"Arial" );
	if ( PopCalVars->HeaderFont == NULL )
		EndDialog( hwnd, -101 );

	PopCalVars->DateFont = CreateFont( lfHeight1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Arial" );
	if ( PopCalVars->DateFont == NULL )
		EndDialog( hwnd, -102 );

	PopCalVars->SelectedFont = CreateFont( lfHeight1, 0, 0, 0, FW_BOLD, 0, 0, 0, 0, 0, 0, 0, 0, L"Arial" );
	if ( PopCalVars->SelectedFont == NULL )
		EndDialog( hwnd, -103 );

	PopCalVars->DisabledFont = CreateFont( lfHeight1, 0, 0, 0, 0, TRUE, 0, 0, 0, 0, 0, 0, 0, L"Arial" );
	if ( PopCalVars->DisabledFont == NULL )
		EndDialog( hwnd, -104 );

	PopCalVars->SmallFont = CreateFont( lfHeight3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Arial" );
	if ( PopCalVars->SmallFont == NULL )
		EndDialog( hwnd, -105 );

	PopCalVars->ButtonFont = CreateFont( lfHeight2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, L"Arial" );
	if ( PopCalVars->ButtonFont == NULL )
		EndDialog( hwnd, -106 );

	SendDlgItemMessage( hwnd, HEADERLINE, WM_SETFONT, (WPARAM) PopCalVars->HeaderFont, TRUE );
	SendDlgItemMessage( hwnd, BACKBUTTON, WM_SETFONT, (WPARAM) PopCalVars->ButtonFont, TRUE );
	SendDlgItemMessage( hwnd, FORWARDBUTTON, WM_SETFONT, (WPARAM) PopCalVars->ButtonFont, TRUE );
	SendDlgItemMessage( hwnd, YEARBACKBUTTON, WM_SETFONT, (WPARAM) PopCalVars->SmallFont, TRUE );
	SendDlgItemMessage( hwnd, YEARFORWARDBUTTON, WM_SETFONT, (WPARAM) PopCalVars->SmallFont, TRUE );
	SendDlgItemMessage( hwnd, IDOK, WM_SETFONT, (WPARAM) PopCalVars->ButtonFont, TRUE );
	SendDlgItemMessage( hwnd, IDCANCEL, WM_SETFONT, (WPARAM) PopCalVars->ButtonFont, TRUE );
	SendDlgItemMessage( hwnd, OTHERBUTTON, WM_SETFONT, (WPARAM) PopCalVars->ButtonFont, TRUE );
	SendDlgItemMessage( hwnd, STATUSLINE, WM_SETFONT, (WPARAM) PopCalVars->SmallFont, TRUE );

	if ( RgbLum( GetSysColor( COLOR_BTNFACE ) ) >= 500 ) {
		PopCalVars->SelectedBoxBgnd = CreateSolidBrush( RGB( 0xb8, 0xb8, 0xff ) );
		PopCalVars->SundayTextColor  = RGB( 0xb0, 0x00, 0x00 );
		PopCalVars->HolidayTextColor = RGB( 0x00, 0x00, 0xc0 );
	}
	else {
		PopCalVars->SelectedBoxBgnd = CreateSolidBrush( RGB( 0x30, 0x30, 0x60 ) );
		PopCalVars->SundayTextColor  = RGB( 0xff, 0x40, 0x40 );
		PopCalVars->HolidayTextColor = RGB( 0x60, 0x80, 0xff );
	}

	if ( PopCalVars->SelectedBoxBgnd == NULL )
		EndDialog( hwnd, -107 );

	PopCalVars->DisabledColor	= GetSysColor( COLOR_GRAYTEXT );

	PopCalVars->KeyboardHook	= SetWindowsHookEx( WH_KEYBOARD, (HOOKPROC) PopCalKeyboardProc, NULL, GetCurrentThreadId() );

	if ( PopCalVars->Flags & POPCAL_FLAG_NO_CANCEL ) {
		EnableWindow( GetDlgItem( hwnd, IDCANCEL ), FALSE );
		ShowWindow( GetDlgItem( hwnd, IDCANCEL ), FALSE );
		// HMENU hMenu = GetSystemMenu( hwnd, FALSE );
		// RemoveMenu( hMenu, SC_CLOSE, MF_BYCOMMAND );
		// DrawMenuBar( hwnd );
	}

	DrawStatusLine( hwnd );

	if ( PopCalVars->Countdown ) {
		PopCalVars->KeystrokeEndsCountdown = FALSE;
		wchar_t shortbuf[SHORT_BUF_LEN];
		wsprintf( shortbuf, L"OK - %d", PopCalVars->Countdown);
		SetDlgItemText( hwnd, IDOK, shortbuf );
		PopCalVars->DoCountdown = TRUE;
		PopCalVars->TimerID = SetTimer( NULL, NULL, 1000, (TIMERPROC) PopCalTimerProc );
	}
	else
		PopCalVars->DoCountdown = FALSE;

	return 0;
}



INT_PTR CALLBACK PopCalDlgProc( HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam )
{
	switch ( Message ) {

	case WM_INITDIALOG :
		PopCalInitializeDialog( hwnd );
		FillDowLine( hwnd );
		DrawPopCalendar( hwnd );
		return TRUE;

	case WM_COMMAND :

		PopCalTerminateTimer();

		switch ( LOWORD ( wParam ) ) {

		case IDOK :
			EndDialog( hwnd, IDOK );
			break;

		case IDCANCEL :
			EndDialog( hwnd, IDCANCEL );
			break;

		case BACKBUTTON :
			BackMonth( hwnd );
			break;

		case FORWARDBUTTON :
			AheadMonth( hwnd );
			break;

		case YEARBACKBUTTON :
			BackYear( hwnd );
			break;

		case YEARFORWARDBUTTON :
			AheadYear( hwnd );
			break;

		case SELECTBACKDAY :
			SelBackDay( hwnd );
			break;

		case SELECTAHEADDAY :
			SelAheadDay( hwnd );
			break;

		case SELECTBACKWEEK :
			SelBackWeek( hwnd );
			break;

		case SELECTAHEADWEEK :
			SelAheadWeek( hwnd );
			break;

		case SELECTBACKMONTH :
			SelBackMonth( hwnd );
			break;

		case SELECTAHEADMONTH :
			SelAheadMonth( hwnd );
			break;

		case SELECTBACKYEAR :
			SelBackYear( hwnd );
			break;

		case SELECTAHEADYEAR :
			SelAheadYear( hwnd );
			break;

		case SELECTWEEKTOP :
			SelWeekTop( hwnd );
			break;

		case SELECTWEEKEND :
			SelWeekEnd( hwnd );
			break;

		case SELECTMINIMUM :
			SelMinimum( hwnd );
			break;

		case SELECTMAXIMUM :
			SelMaximum( hwnd );
			break;

		case SELECTINITIAL :
			SelInit( hwnd );
			break;

		case OTHERBUTTON :
			ChangeLeftColumn( hwnd );
			break;

		case STATUSLINE :
			if ( HIWORD( wParam ) == STN_CLICKED ) {
				PopCalVars->View = PopCalVars->Selected;
				DrawPopCalendar( hwnd );
			}
			break;

		default :
			if ( ( LOWORD( wParam ) >= DATEBOX0 ) && ( LOWORD( wParam ) <= ( DATEBOX0 + POPCAL_NUM_BOXES - 1 ) ) && ( HIWORD( wParam ) == STN_CLICKED ) )
				ChangeSelection( hwnd, LOWORD( wParam ) - DATEBOX0 );
			else if ( ( LOWORD( wParam ) >= DATEBOX0 ) && ( LOWORD( wParam ) <= ( DATEBOX0 + POPCAL_NUM_BOXES - 1 ) ) && ( HIWORD( wParam ) == STN_DBLCLK ) )
				QuickExit( hwnd, LOWORD( wParam ) - DATEBOX0 );
		}

	case WM_CTLCOLORSTATIC : {

		HDC hdcStatic = (HDC) wParam;
		int ID = GetDlgCtrlID( (HWND) lParam );

		if ( ( ID >= DATEBOX0 ) && ( ID < ( DATEBOX0 + POPCAL_NUM_BOXES ) ) ) {
			int box = ID - DATEBOX0;

			if ( PopCalVars->Box[box] & POPCAL_BOX_SELECTED ) {
				SetBkMode( hdcStatic, TRANSPARENT );
				if ( PopCalVars->Box[box] & POPCAL_BOX_HOLIDAY )
					SetTextColor( hdcStatic, PopCalVars->HolidayTextColor );
				else if ( PopCalVars->Box[box] & POPCAL_BOX_SUNDAY )
					SetTextColor( hdcStatic, PopCalVars->SundayTextColor );
				return (LONG) PopCalVars->SelectedBoxBgnd;
			}
			else if ( PopCalVars->Box[box] & POPCAL_BOX_DISABLED ) {
				SetTextColor( hdcStatic, PopCalVars->DisabledColor );
				SetBkMode( hdcStatic, TRANSPARENT );
				return (LONG) ( COLOR_BTNFACE + 1 );
			}
			else if ( PopCalVars->Box[box] & POPCAL_BOX_HOLIDAY ) {
				SetBkMode( hdcStatic, TRANSPARENT );
				SetTextColor( hdcStatic, PopCalVars->HolidayTextColor );
				return (LONG) ( COLOR_BTNFACE + 1 );
			}
			else if ( PopCalVars->Box[box] & POPCAL_BOX_SUNDAY ) {
				SetTextColor( hdcStatic, PopCalVars->SundayTextColor );
				SetBkMode( hdcStatic, TRANSPARENT );
				return (LONG) ( COLOR_BTNFACE + 1 );
			}
		}
		else if ( ( ID == ( DOWBOX0 + ( ( 7 - LeftColumnDow ) % 7 ) ) ) ) {
			SetTextColor( hdcStatic, PopCalVars->SundayTextColor );
			SetBkMode( hdcStatic, TRANSPARENT );
			return (LONG) ( COLOR_BTNFACE + 1 );
			}
		}
		return FALSE;

	case WM_DESTROY :
		PopCalDeleteObjects();
		FreeHolidayDefs();
		return 0;

	default:
		return FALSE;
	}

	return TRUE;
}



BOOL ResolveMaskedSelection()
{
	if ( !( PopCalVars->DowMask & 0x7f ) && !( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS ) )
		return TRUE;
	else if ( ( PopCalVars->DowMask & 0x7f ) == 0x7f )
		return FALSE;


	long int minjul		= Julian( PopCalVars->MinDate );
	long int maxjul		= Julian( PopCalVars->MaxDate );
	long int julian		= Julian( PopCalVars->Selected );
	long int upjul		= julian;
	long int downjul	= julian;


	if ( PopCalVars->Flags & POPCAL_FLAG_DISABLE_HOLIDAYS ) {

		if ( ( ( PopCalVars->DowMask & ( 1 << ( julian % 7 ) ) ) == 0 ) && !PopCalIsHoliday( julian ) )
			return TRUE;

		while ( ( PopCalVars->DowMask & ( 1 << ( upjul % 7 ) ) ) || PopCalIsHoliday( upjul ) )
			upjul ++;

		while ( ( PopCalVars->DowMask & ( 1 << ( downjul % 7 ) ) ) || PopCalIsHoliday( downjul ) )
			downjul--;
	}
	else {

		if ( ( PopCalVars->DowMask & ( 1 << ( julian % 7 ) ) ) == 0 )
			return TRUE;

		while ( PopCalVars->DowMask & ( 1 << ( upjul % 7 ) ) )
			upjul ++;

		while ( PopCalVars->DowMask & ( 1 << ( downjul % 7 ) ) )
			downjul--;
	}

	if ( ( downjul < minjul ) && ( upjul > maxjul ) )
		return FALSE;

	if ( downjul < minjul ) {
		MakeDate( upjul, PopCalVars->Selected );
		return TRUE;
	}

	if ( upjul > maxjul ) {
		MakeDate( downjul, PopCalVars->Selected );
		return TRUE;
	}

	if ( ( PopCalVars->Flags & POPCAL_FLAG_PREFER_MASK ) == POPCAL_FLAG_PREFER_EARLIER ) {
		MakeDate( downjul, PopCalVars->Selected );
		return TRUE;
	}

	if ( ( PopCalVars->Flags & POPCAL_FLAG_PREFER_MASK ) == POPCAL_FLAG_PREFER_LATER ) {
		MakeDate( upjul, PopCalVars->Selected );
		return TRUE;
	}

	if ( ( upjul - julian ) <= ( julian - downjul ) ) {
		MakeDate( upjul, PopCalVars->Selected );
		return TRUE;
	}

	MakeDate( downjul, PopCalVars->Selected );
	return TRUE;
}



DLLExports INT WINAPI f_pickdate( LPTSTR String )
{
	if ( !String )
		return 1;


	_PopCalVars Vars;

	memset( &Vars, 0x00, sizeof( _PopCalVars ) );
	PopCalVars		= &Vars;


	GetNow( FALSE );
	GetLocalDateFormat();
	Vars.Selected.year	= Now.wYear;
	Vars.Selected.month	= Now.wMonth;
	Vars.Selected.day	= Now.wDay;

	Vars.MinDate.year  = Epoch;
	Vars.MinDate.month = 1;
	Vars.MinDate.day   = 1;
	Vars.MaxDate.year  = Ultima;
	Vars.MaxDate.month = 12;
	Vars.MaxDate.day   = 31;


	int rv = -1, fmt = 0x100, errfmt = 0x100;
	int DialogReturn	= 0;


	ParseArgs( String, PARSE_BREAK_COMMAS );


	if ( arglen[0] ) {
		Vars.Caption = (LPTSTR) malloc( ( wcslen( arg[0] ) + 1 ) * sizeof( wchar_t ) );

		if ( !Vars.Caption ) {
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Error allocating caption buffer\r\n" );
			goto ErrorExit;
		}

		wcscpy_s( Vars.Caption, wcslen( arg[0] ) + 1, arg[0] );
	}

	if ( arglen[2] ) {		// Third parameter:  Date format
		fmt = ParseInt( arg[2] );

		if ( fmt < 0 ) {
			_wcsupr_s( arg[2], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid format \"%s\"\r\n", arg[2] );
			goto ErrorExit;
		}
	}

	if ( arglen[7] ) {		// Eighth parameter:  Disabled days-of-the-week
		Vars.DowMask = ParseDowMask( arg[7] );

		if ( ( Vars.DowMask < 0 ) || ( Vars.DowMask > 126 ) ) {
			_wcsupr_s( arg[7], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid days-of-the-week mask \"%s\"\r\n", arg[7] );
			goto ErrorExit;
		}
	}

	if ( arglen[5] ) {		// Sixth parameter:  Misc program flags
		Vars.Flags = ParseInt( arg[5] );

		if ( Vars.Flags < 0 ) {
			_wcsupr_s( arg[5], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid flags \"%s\"\r\n", arg[5] );
			goto ErrorExit;
		}
	}

	if ( arglen[1] ) {		// Second parameter:  Initial (selected) date
		if ( ParseDate( arg[1], Vars.Selected, fmt ) < 0 ) {
			_wcsupr_s( arg[1], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid initial date \"%s\"\r\n", arg[1] );
			goto ErrorExit;
		}
	}

	if ( arglen[3] ) {		// Fourth parameter:  Minimum date
		if ( IsSign( arg[3][0] ) && iswdigit( arg[3][1] ) ) {
			long int n = Julian( Vars.Selected ) + wcstol( arg[3], NULL, 10 );

			if ( ( n < MinimumJulian ) || ( n > MaximumJulian ) ) {
				_wcsupr_s( arg[3], MAX_ARG_LEN );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Invalid minimum date \"%s\" (offset)\r\n", arg[3] );
				goto ErrorExit;
			}

			MakeDate( n, Vars.MinDate );
		}
		else if ( ParseDate( arg[3], Vars.MinDate, fmt ) < 0 ) {
			_wcsupr_s( arg[3], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid minimum date \"%s\"\r\n", arg[3] );
			goto ErrorExit;
		}
	}

	if ( arglen[4] ) {		// Fifth parameter:  Maximum date
		if ( IsSign( arg[4][0] ) && iswdigit( arg[4][1] ) ) {
			long int n = Julian( Vars.Selected ) + wcstol( arg[4], NULL, 10 );

			if ( ( n < MinimumJulian ) || ( n > MaximumJulian ) ) {
				_wcsupr_s( arg[4], MAX_ARG_LEN );
				DisplayErrorHeader();
				Qprintf( ERRHANDLE, L"Invalid maximum date \"%s\" (offset)\r\n", arg[4] );
				goto ErrorExit;
			}

			MakeDate( n, Vars.MaxDate );
		}
		else if ( ParseDate( arg[4], Vars.MaxDate, fmt ) < 0 ) {
			_wcsupr_s( arg[4], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid maximum date \"%s\"\r\n", arg[4] );
			goto ErrorExit;
		}
	}

	if ( arglen[6] ) {		// Seventh parameter:  Left column day-of-week
		int lc = ParseDow( arg[6] );
		if ( lc < 0 ) {
			_wcsupr_s( arg[6], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid left-column day value \"%s\"\r\n", arg[6] );
			goto ErrorExit;
		}
		else
			LeftColumnDow = lc % 7;
	}

	if ( arglen[8] ) {		// Ninth parameter:  Timeout value
		Vars.Countdown = ParseInt( arg[8] );

		if ( ( Vars.Countdown < 0 ) || ( Vars.Countdown > 600 ) ) {
			_wcsupr_s( arg[8], MAX_ARG_LEN );
			DisplayErrorHeader();
			Qprintf( ERRHANDLE, L"Invalid timeout \"%s\"\r\n", arg[8] );
			goto ErrorExit;
		}
	}


	if ( Vars.Flags & POPCAL_FLAG_RESTRICT ) {
		if ( Vars.MinDate.year < 1980) {
			Vars.MinDate.year  = 1980;
			Vars.MinDate.month = 1;
			Vars.MinDate.day   = 1;
		}

		if ( Vars.MaxDate.year > 2099 ) {
			Vars.MaxDate.year  = 2099;
			Vars.MaxDate.month = 12;
			Vars.MaxDate.day   = 31;
		}
	}

	if ( Vars.Flags & POPCAL_FLAG_DISABLE_HOLIDAYS )
		Vars.Flags &= ~POPCAL_FLAG_HIDE_HOLIDAYS;

	if ( ( fmt & 0xff ) <= 4 )
		errfmt = fmt;
	
	if ( Julian( Vars.MinDate ) > Julian( Vars.MaxDate ) ) {
		OutputFormattedDate( String + 0, Vars.MinDate, errfmt );
		OutputFormattedDate( String + 128, Vars.MaxDate, errfmt );
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Invalid date range %s - %s\r\n", String + 0, String + 128 );
		goto ErrorExit;
	}
	
	if ( ( arglen[1] == 0 ) && ( arglen[3] > 0 ) && ( Julian( Vars.Selected ) < Julian( Vars.MinDate ) ) )
		Vars.Selected = Vars.MinDate;
	else if ( ( arglen[1] == 0 ) && ( arglen[4] > 0 ) && ( Julian( Vars.Selected ) > Julian( Vars.MaxDate ) ) )
		Vars.Selected = Vars.MaxDate;

	if ( ( Julian( Vars.Selected ) < Julian( Vars.MinDate ) ) || ( Julian( Vars.Selected ) > Julian( Vars.MaxDate ) ) ) {
		OutputFormattedDate( String + 0, Vars.Selected, errfmt );
		OutputFormattedDate( String + 128, Vars.MinDate, errfmt );
		OutputFormattedDate( String + 256, Vars.MaxDate, errfmt );
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"Date %s not in range %s - %s\r\n", String + 0, String + 128, String + 256 );
		goto ErrorExit;
	}

	FreeHolidayDefs();

	if ( QcalReadHolidaysFile() <= 0 )
		QcalDefaultHolidays();

	if ( !ResolveMaskedSelection() ) {
		OutputFormattedDate( String + 0, Vars.MinDate, errfmt );
		OutputFormattedDate( String + 128, Vars.MaxDate, errfmt );
		DisplayErrorHeader();
		Qprintf( ERRHANDLE, L"No unmasked days in range %s - %s\r\n", String + 0, String + 128 );
		goto ErrorExit;
	}


	Vars.View = Vars.Selected;
	Vars.InitialDate = Vars.Selected;

	HoldSignals();
	DialogReturn = (int) DialogBox( MyDLLHandle, MAKEINTRESOURCE( DLG_POPCAL ), GetForegroundWindow(), PopCalDlgProc );

	switch ( DialogReturn ) {

	case IDOK :
		// return selected date
		OutputFormattedDate( String, Vars.Selected, fmt );
		rv = 0;
		break;

	case IDCANCEL :
		// canceled by user; return info string
		wcscpy_s( String, MAX_ARG_LEN, L"*CANCEL*" );
		rv = 0;
		break;

	default:
		// dialog failed?
		wsprintf( String, L"Dialog error %d", DialogReturn );
		rv = -1;
	}


ErrorExit:

	if ( Vars.Caption )
		free( Vars.Caption );

	PopCalVars = NULL;

	EnableSignals();
	return rv;
}


#endif	//	POPCAL
